<?php
/**
 * @link https://www.saskaita123.lt
 * @package Saskaita123Plugin
 *
 * Class Description: Create S123Invoice
 */

namespace S123\Includes\Woocommerce;

use S123\Includes\Base\S123_BaseController;

if (!defined('ABSPATH')) exit;

class S123_Invoice extends S123_BaseController
{
    /*
     * Order data object
     */
    protected $order;

    /*
    * Build array for API products
    */
    private function s123_invoiceProducts($item, $product): array
    {
        $invoiceProduct = [];
        $invoiceProduct["title"] = $item->get_name();
        $invoiceProduct["description"] = substr($product->get_short_description(), 0, 200);
        $invoiceProduct["price"] = number_format(wc_get_price_excluding_tax($product), 4, '.', '');
        $invoiceProduct["quantity"] = $item->get_quantity();
        $totalProduct = wc_get_price_excluding_tax($product) * $item->get_quantity();
        $invoiceProduct["total"] = number_format($totalProduct, 4, '.', '');
        $invoiceProduct["unit_id"] = $this->s123_get_option('api_units');
        if ($this->s123_get_option('use_invoice_vats') === true && (wc_get_price_excluding_tax($product) !== wc_get_price_including_tax($product))) {
            $invoiceProduct["company_vat_id"] = $this->s123_get_option('api_vats');
        }

        return $invoiceProduct;
    }

    /*
    * Add shipping data as product to API
    */
    private function s123_addShippingToPayment(): array
    {
        $invoiceShipping = [];
        $invoiceShipping["title"] = __('Shipping', 'woocommerce');
        $invoiceShipping["price"] = number_format($this->order->get_shipping_total(), 4, '.', '');
        $invoiceShipping["quantity"] = 1;
        $invoiceShipping["total"] = number_format($this->order->get_shipping_total(), 4, '.', '');
        $invoiceShipping["unit_id"] = $this->s123_get_option('api_units');

        if ($this->s123_get_option('use_invoice_vats') === true && $this->order->get_shipping_tax()) {
            $invoiceShipping["company_vat_id"] = $this->s123_get_option('api_vats');
        }

        return $invoiceShipping;
    }

    /*
    * Build array for API client data
    */
    private function s123_invoiceClient(): array
    {
        $client = [];
        $clientCode = get_post_meta($this->order->get_id(), '_billing_company_code', true);

        // check if order has set company name and company code
        if ($this->order->get_billing_company() && $clientCode) {
            $client["name"] = $this->order->get_billing_company();
        } else {
            $client["name"] = $this->order->get_billing_first_name() . ' ' . $this->order->get_billing_last_name();
        }

        // check if order has set company code
        if ($clientCode) {
            $client["code"] = $clientCode;
            $client["code_type"] = 'company';
        } else {
            $client["code_type"] = 'personal';
        }

        $clientVat = get_post_meta($this->order->get_id(), '_billing_company_vat_code', true);
        if ($clientVat) {
            $client["vat_code"] = $clientVat;
        }

        $client["address"] = $this->order->get_billing_address_1() ?: $this->order->get_billing_address_2();
        $client["email"] = $this->order->get_billing_email();
        $client["phone"] = $this->order->get_billing_phone();

        return $client;
    }

    /*
    * Build array for payments
    */
    private function s123_invoicePayments(): array
    {
        $invoicePayments = [];

        $invoicePayments["total"] = number_format($this->order->get_total(), 4, '.', '');
        $invoicePayments["type"] = 'transfer';
        $invoicePayments["date"] = $this->order->get_date_paid()->date('Y-m-d');

        return $invoicePayments;
    }

    /*
    * Build API invoice data
    */
    public function s123_buildInvoice($order): array
    {
        $this->order = $order;
        $products = [];
        foreach ($this->order->get_items() as $item) {
            // Get the product object
            $product = $item->get_product();
            $products[] = $this->s123_invoiceProducts($item, $product);
        }

        if ($this->order->get_shipping_total() !== '0') {
            $products[] = $this->s123_addShippingToPayment();
        }

        $client = $this->s123_invoiceClient();

        $payments[] = $this->s123_invoicePayments();

        return [
            "type" => "simple",
            "series_id" => $this->s123_get_option('api_series'),
            "activity_id" => $this->s123_get_option('use_activities') ? $this->s123_get_option('api_activities') : null,
            "date" => $this->order->get_date_created()->date('Y-m-d'),
            "discount" => (string)$this->order->get_total_discount() ?: '0.00',
            "discount_type" => 'fixed',
            "discount_value" => (string)$this->order->get_total_discount() ?: '0.00',
            "total" =>  number_format($this->order->get_total(), 4, '.', ''),
            "issued_by" => $this->s123_get_option('invoice_issued_by'),
            "issued_to" => $client["name"],
            "products" => $products,
            "client" => $client,
            "banks" => $this->s123_get_option('api_banks'),
            "payments" => $payments,
        ];
    }
}